type __ = Obj.t
let __ = let rec f _ = Obj.repr f in Obj.repr f

type ('a, 'b) sum =
| Inl of 'a
| Inr of 'b

(** val fst : ('a1 * 'a2) -> 'a1 **)

let fst = function
| (x, y) -> x

(** val snd : ('a1 * 'a2) -> 'a2 **)

let snd = function
| (x, y) -> y

(** val app : 'a1 list -> 'a1 list -> 'a1 list **)

let rec app l m =
  match l with
  | [] -> m
  | a :: l1 -> a :: (app l1 m)

(** val id : 'a1 -> 'a1 **)

let id x =
  x

type 'a sig0 =
  'a
  (* singleton inductive, whose constructor was exist *)

type 'world action = 'world -> 'world

type ('input, 'world) process = ('input, 'world) __process Lazy.t
and ('input, 'world) __process =
| Step of ('input -> 'world action * ('input, 'world) process)

type ('input, 'world) stackWorld =
| StackDone
| StackPush of 'input * ('input, 'world) stackWorld
| StackLift of 'world action * ('input, 'world) stackWorld

type ('input, 'world) stackProcess =
  ('input, ('input, 'world) stackWorld) process

(** val stepStackProcess :
    (('a1, 'a2) stackWorld * ('a1, 'a2) stackProcess) -> 'a2 action * ('a1,
    'a2) stackProcess **)

let rec stepStackProcess = function
| (s, x) ->
  (match s with
   | StackDone -> (id, x)
   | StackPush (m, sw') ->
     let Step pf = Lazy.force x in
     let sap1 = pf m in
     let sw = fst sap1 StackDone in
     let p = stepStackProcess (sw, (snd sap1)) in
     let (a2, s0) = p in
     let p0 = stepStackProcess (sw', s0) in
     let (a3, s1) = p0 in ((fun w -> a2 (a3 w)), s1)
   | StackLift (a, sw') ->
     let p0 = stepStackProcess (sw', x) in
     let (a2, s0) = p0 in ((fun w -> a (a2 w)), s0))

(** val runStackProcess : ('a1, 'a2) stackProcess -> ('a1, 'a2) process **)

let rec runStackProcess p =
  lazy (Step (fun i ->
    let Step pf = Lazy.force p in
    let sap1 = pf i in
    let sw = fst sap1 StackDone in
    let stepped = stepStackProcess (sw, (snd sap1)) in
    ((fst stepped), (runStackProcess (snd stepped)))))

(** val map : ('a1 -> 'a2) -> 'a1 list -> 'a2 list **)

let rec map f = function
| [] -> []
| a :: t -> (f a) :: (map f t)

(** val flat_map : ('a1 -> 'a2 list) -> 'a1 list -> 'a2 list **)

let rec flat_map f = function
| [] -> []
| x :: t -> app (f x) (flat_map f t)

(** val fold_right : ('a2 -> 'a1 -> 'a1) -> 'a1 -> 'a2 list -> 'a1 **)

let rec fold_right f a0 = function
| [] -> a0
| b :: t -> f b (fold_right f a0 t)

(** val filter : ('a1 -> bool) -> 'a1 list -> 'a1 list **)

let rec filter f = function
| [] -> []
| x :: l0 -> if f x then x :: (filter f l0) else filter f l0

(** val find : ('a1 -> bool) -> 'a1 list -> 'a1 option **)

let rec find f = function
| [] -> None
| x :: tl -> if f x then Some x else find f tl

(** val string_dec : char list -> char list -> bool **)

let rec string_dec s s0 =
  match s with
  | [] ->
    (match s0 with
     | [] -> true
     | a::s1 -> false)
  | a::s1 ->
    (match s0 with
     | [] -> false
     | a0::s2 -> if (=) a a0 then string_dec s1 s2 else false)

(** val append : char list -> char list -> char list **)

let rec append s1 s2 =
  match s1 with
  | [] -> s2
  | c::s1' -> c::(append s1' s2)

type ('world, 't) abortable =
| Started of 'world action
| Done of 't

type httpStatus =
| HttpOk
| HttpPreconditionFailed
| HttpUnrecognizedCode

type httpResponse = { httpResponseStatus : httpStatus;
                      httpResponseStatusText : char list;
                      httpResponseProtocol : char list;
                      httpResponseHeader : (char list * char list) list;
                      httpResponseBody : char list }

(** val httpResponseStatus : httpResponse -> httpStatus **)

let httpResponseStatus x = x.httpResponseStatus

(** val httpResponseBody : httpResponse -> char list **)

let httpResponseBody x = x.httpResponseBody

type ('input, 'world) systemActions = { consoleErr : (char list -> 'world
                                                     action);
                                        consoleIn : ((char list -> 'input) ->
                                                    'world action);
                                        consoleOut : (char list -> 'world
                                                     action);
                                        exit : (Big.big_int -> 'world action);
                                        getArgv : ((char list list -> 'input)
                                                  -> 'world action);
                                        getNanosecs : ((Big.big_int ->
                                                      'input) -> 'world
                                                      action);
                                        getRandomness : (Big.big_int ->
                                                        (char list -> 'input)
                                                        -> 'world action);
                                        httpPOST : (char list ->
                                                   (char list * char list)
                                                   list -> (('world,
                                                   httpResponse) abortable ->
                                                   'input) -> 'world action);
                                        setDebug : (Big.big_int -> 'world
                                                   action);
                                        sleepNanosecs : (Big.big_int ->
                                                        'input -> 'world
                                                        action) }

(** val consoleIn :
    ('a1, 'a2) systemActions -> (char list -> 'a1) -> 'a2 action **)

let consoleIn x = x.consoleIn

(** val consoleOut : ('a1, 'a2) systemActions -> char list -> 'a2 action **)

let consoleOut x = x.consoleOut

(** val httpPOST :
    ('a1, 'a2) systemActions -> char list -> (char list * char list) list ->
    (('a2, httpResponse) abortable -> 'a1) -> 'a2 action **)

let httpPOST x = x.httpPOST

(** val storageURI : char list **)

let storageURI =
  'h'::('t'::('t'::('p'::('s'::(':'::('/'::('/'::('a'::('n'::('d'::('e'::('r'::('s'::('k'::('.'::('s'::('c'::('r'::('i'::('p'::('t'::('s'::('.'::('m'::('i'::('t'::('.'::('e'::('d'::('u'::('/'::('p'::('w'::('m'::('g'::('r'::('/'::('s'::('t'::('o'::('r'::('a'::('g'::('e'::[]))))))))))))))))))))))))))))))))))))))))))))

(** val storageSet :
    (char list -> (char list * char list) list -> (('a1, httpResponse)
    abortable -> 'a3) -> 'a2 action) -> char list -> char list -> char list
    -> (('a1, (char list option, httpResponse) sum) abortable -> 'a3) -> 'a2
    action **)

let storageSet httpPOST0 id0 old new0 cb =
  httpPOST0 (append storageURI ('/'::('s'::('e'::('t'::[])))))
    ((('i'::('d'::[])), id0) :: ((('o'::('l'::('d'::[]))),
    old) :: ((('n'::('e'::('w'::[]))), new0) :: []))) (fun ra ->
    match ra with
    | Started abort -> cb (Started abort)
    | Done r ->
      (match r.httpResponseStatus with
       | HttpOk -> cb (Done (Inl None))
       | HttpPreconditionFailed -> cb (Done (Inl (Some r.httpResponseBody)))
       | HttpUnrecognizedCode -> cb (Done (Inr r))))

type uiInput =
| UiConsoleIn of char list
| UiDecrypted of char list

(** val split : char -> char list -> char list list **)

let rec split sep = function
| [] -> []
| c::s' ->
  if (=) c sep
  then [] :: (split sep s')
  else (match split sep s' with
        | [] -> (c::[]) :: []
        | w :: ws -> (c::w) :: ws)

(** val newline : char **)

let newline =
  '\n'

(** val dump : (char list * char list) list -> char list **)

let dump pws =
  fold_right append []
    (map (fun p ->
      append (fst p) (append (' '::[]) (append (snd p) (newline::[])))) pws)

(** val load : char list -> (char list * char list) list **)

let rec load s =
  flat_map (fun l ->
    match split ' ' l with
    | [] -> []
    | account :: l0 ->
      (match l0 with
       | [] -> []
       | password :: l1 ->
         (match l1 with
          | [] -> (account, password) :: []
          | s0 :: l2 -> []))) (split newline s)

(** val uiLoopBody :
    (char list -> 'a1 action) -> (char list -> 'a1 action) ->
    ((char list * char list) list -> (uiInput, 'a1) process) ->
    (char list * char list) list -> uiInput -> 'a1 action * (uiInput, 'a1)
    process **)

let uiLoopBody consoleOut0 encrypt uiLoop0 pws = function
| UiConsoleIn s ->
  (match split ' ' s with
   | [] ->
     ((consoleOut0
        ('u'::('n'::('r'::('e'::('c'::('o'::('g'::('n'::('i'::('z'::('e'::('d'::(' '::('c'::('o'::('m'::('m'::('a'::('n'::('d'::[]))))))))))))))))))))),
       (uiLoop0 pws))
   | comm :: ls ->
     if string_dec comm ('g'::('e'::('t'::[])))
     then (match ls with
           | [] ->
             if string_dec comm ('s'::('e'::('t'::[])))
             then (match ls with
                   | [] ->
                     ((consoleOut0
                        ('u'::('n'::('r'::('e'::('c'::('o'::('g'::('n'::('i'::('z'::('e'::('d'::(' '::('c'::('o'::('m'::('m'::('a'::('n'::('d'::[]))))))))))))))))))))),
                       (uiLoop0 pws))
                   | account :: l ->
                     (match l with
                      | [] ->
                        ((consoleOut0
                           ('u'::('n'::('r'::('e'::('c'::('o'::('g'::('n'::('i'::('z'::('e'::('d'::(' '::('c'::('o'::('m'::('m'::('a'::('n'::('d'::[]))))))))))))))))))))),
                          (uiLoop0 pws))
                      | password :: l0 ->
                        (match l0 with
                         | [] ->
                           let pws' = (account,
                             password) :: (filter (fun p ->
                                            if string_dec account (fst p)
                                            then false
                                            else true) pws)
                           in
                           ((encrypt (dump pws')), (uiLoop0 pws'))
                         | s0 :: l1 ->
                           ((consoleOut0
                              ('u'::('n'::('r'::('e'::('c'::('o'::('g'::('n'::('i'::('z'::('e'::('d'::(' '::('c'::('o'::('m'::('m'::('a'::('n'::('d'::[]))))))))))))))))))))),
                             (uiLoop0 pws)))))
             else ((consoleOut0
                     ('u'::('n'::('r'::('e'::('c'::('o'::('g'::('n'::('i'::('z'::('e'::('d'::(' '::('c'::('o'::('m'::('m'::('a'::('n'::('d'::[]))))))))))))))))))))),
                    (uiLoop0 pws))
           | account :: l ->
             (match l with
              | [] ->
                (match find (fun p ->
                         if string_dec account (fst p) then true else false)
                         pws with
                 | Some p ->
                   let (s0, password) = p in
                   ((consoleOut0 password), (uiLoop0 pws))
                 | None ->
                   ((consoleOut0
                      ('a'::('c'::('c'::('o'::('u'::('n'::('t'::(' '::('n'::('o'::('t'::(' '::('f'::('o'::('u'::('n'::('d'::[])))))))))))))))))),
                     (uiLoop0 pws)))
              | s0 :: l0 ->
                if string_dec comm ('s'::('e'::('t'::[])))
                then (match ls with
                      | [] ->
                        ((consoleOut0
                           ('u'::('n'::('r'::('e'::('c'::('o'::('g'::('n'::('i'::('z'::('e'::('d'::(' '::('c'::('o'::('m'::('m'::('a'::('n'::('d'::[]))))))))))))))))))))),
                          (uiLoop0 pws))
                      | account0 :: l1 ->
                        (match l1 with
                         | [] ->
                           ((consoleOut0
                              ('u'::('n'::('r'::('e'::('c'::('o'::('g'::('n'::('i'::('z'::('e'::('d'::(' '::('c'::('o'::('m'::('m'::('a'::('n'::('d'::[]))))))))))))))))))))),
                             (uiLoop0 pws))
                         | password :: l2 ->
                           (match l2 with
                            | [] ->
                              let pws' = (account0,
                                password) :: (filter (fun p ->
                                               if string_dec account0 (fst p)
                                               then false
                                               else true) pws)
                              in
                              ((encrypt (dump pws')), (uiLoop0 pws'))
                            | s1 :: l3 ->
                              ((consoleOut0
                                 ('u'::('n'::('r'::('e'::('c'::('o'::('g'::('n'::('i'::('z'::('e'::('d'::(' '::('c'::('o'::('m'::('m'::('a'::('n'::('d'::[]))))))))))))))))))))),
                                (uiLoop0 pws)))))
                else ((consoleOut0
                        ('u'::('n'::('r'::('e'::('c'::('o'::('g'::('n'::('i'::('z'::('e'::('d'::(' '::('c'::('o'::('m'::('m'::('a'::('n'::('d'::[]))))))))))))))))))))),
                       (uiLoop0 pws))))
     else if string_dec comm ('s'::('e'::('t'::[])))
          then (match ls with
                | [] ->
                  ((consoleOut0
                     ('u'::('n'::('r'::('e'::('c'::('o'::('g'::('n'::('i'::('z'::('e'::('d'::(' '::('c'::('o'::('m'::('m'::('a'::('n'::('d'::[]))))))))))))))))))))),
                    (uiLoop0 pws))
                | account :: l ->
                  (match l with
                   | [] ->
                     ((consoleOut0
                        ('u'::('n'::('r'::('e'::('c'::('o'::('g'::('n'::('i'::('z'::('e'::('d'::(' '::('c'::('o'::('m'::('m'::('a'::('n'::('d'::[]))))))))))))))))))))),
                       (uiLoop0 pws))
                   | password :: l0 ->
                     (match l0 with
                      | [] ->
                        let pws' = (account,
                          password) :: (filter (fun p ->
                                         if string_dec account (fst p)
                                         then false
                                         else true) pws)
                        in
                        ((encrypt (dump pws')), (uiLoop0 pws'))
                      | s0 :: l1 ->
                        ((consoleOut0
                           ('u'::('n'::('r'::('e'::('c'::('o'::('g'::('n'::('i'::('z'::('e'::('d'::(' '::('c'::('o'::('m'::('m'::('a'::('n'::('d'::[]))))))))))))))))))))),
                          (uiLoop0 pws)))))
          else ((consoleOut0
                  ('u'::('n'::('r'::('e'::('c'::('o'::('g'::('n'::('i'::('z'::('e'::('d'::(' '::('c'::('o'::('m'::('m'::('a'::('n'::('d'::[]))))))))))))))))))))),
                 (uiLoop0 pws)))
| UiDecrypted s -> (id, (uiLoop0 (load s)))

(** val uiLoop :
    (char list -> 'a1 action) -> (char list -> 'a1 action) ->
    (char list * char list) list -> (uiInput, 'a1) process **)

let rec uiLoop consoleOut0 encrypt pws =
  lazy (Step (uiLoopBody consoleOut0 encrypt (uiLoop consoleOut0 encrypt)
               pws))

(** val ui :
    (char list -> 'a1 action) -> (char list -> 'a1 action) -> (uiInput, 'a1)
    process **)

let ui consoleOut0 encrypt =
  uiLoop consoleOut0 encrypt []

type 'world0 netInput =
| NetReceived of ('world0, (char list option, httpResponse) sum) abortable
| NetEncrypted of char list

(** val storageId : char list **)

let storageId =
  'f'::('o'::('o'::[]))

(** val net :
    (char list -> (char list * char list) list -> (('a1, httpResponse)
    abortable -> 'a1 netInput) -> 'a2 action) -> (char list -> 'a2 action) ->
    ('a1 netInput, 'a2) process **)

let rec net httpPOST0 decrypt =
  lazy (Step (fun i ->
    match i with
    | NetReceived a ->
      (match a with
       | Started abort -> (id, (net httpPOST0 decrypt))
       | Done s0 ->
         (match s0 with
          | Inl o ->
            (match o with
             | Some s -> ((decrypt s), (net httpPOST0 decrypt))
             | None -> (id, (net httpPOST0 decrypt)))
          | Inr h -> (id, (net httpPOST0 decrypt))))
    | NetEncrypted s ->
      ((storageSet httpPOST0 storageId [] s (fun x -> NetReceived x)),
        (net httpPOST0 decrypt))))

(** val getStep :
    ('a1, 'a2) process -> 'a1 -> 'a2 action * ('a1, 'a2) process **)

let getStep p =
  let Step f = Lazy.force p in f

type 'world input =
| PwMgrConsoleIn of char list
| PwMgrNetInput of 'world netInput
| PwMgrEncrypt of char list
| PwMgrDecrypt of char list

(** val pwMgrLoopBody :
    ('a1 input, 'a1) systemActions -> ((uiInput, ('a1 input, 'a1) stackWorld)
    process -> ('a1 netInput, ('a1 input, 'a1) stackWorld) process -> ('a1
    input, 'a1) stackProcess) -> (uiInput, ('a1 input, 'a1) stackWorld)
    process -> ('a1 netInput, ('a1 input, 'a1) stackWorld) process -> 'a1
    input -> ('a1 input, 'a1) stackWorld action * ('a1 input, 'a1)
    stackProcess **)

let pwMgrLoopBody sys pwMgrLoop0 ui0 net0 = function
| PwMgrConsoleIn s ->
  let (a, ui') = getStep ui0 (UiConsoleIn s) in
  ((fun w -> a (StackLift ((sys.consoleIn (fun x -> PwMgrConsoleIn x)), w))),
  (pwMgrLoop0 ui' net0))
| PwMgrNetInput i' ->
  let (a, net') = getStep net0 i' in (a, (pwMgrLoop0 ui0 net'))
| PwMgrEncrypt s ->
  let (a, net') = getStep net0 (NetEncrypted s) in (a, (pwMgrLoop0 ui0 net'))
| PwMgrDecrypt s ->
  let (a, ui') = getStep ui0 (UiDecrypted s) in (a, (pwMgrLoop0 ui' net0))

(** val pwMgrLoop :
    ('a1 input, 'a1) systemActions -> (uiInput, ('a1 input, 'a1) stackWorld)
    process -> ('a1 netInput, ('a1 input, 'a1) stackWorld) process -> ('a1
    input, 'a1) stackProcess **)

let rec pwMgrLoop sys ui0 net0 =
  lazy (Step (pwMgrLoopBody sys (pwMgrLoop sys) ui0 net0))

(** val wrap_ui :
    ('a1 input, 'a1) systemActions -> (__ -> (char list -> __ action) ->
    (char list -> __ action) -> (uiInput, __) process) -> (uiInput, ('a1
    input, 'a1) stackWorld) process **)

let wrap_ui sys ui0 =
  Obj.magic ui0 __ (fun s x -> StackLift ((sys.consoleOut s), x)) (fun s x ->
    StackPush ((PwMgrEncrypt s), x))

(** val wrap_net :
    ('a1 input, 'a1) systemActions -> (__ -> __ -> (char list ->
    (char list * char list) list -> ((__, httpResponse) abortable -> __
    netInput) -> __ action) -> (char list -> __ action) -> (__ netInput, __)
    process) -> ('a1 netInput, ('a1 input, 'a1) stackWorld) process **)

let wrap_net sys net0 =
  Obj.magic net0 __ __ (fun uri data cb x -> StackLift
    ((sys.httpPOST uri data (fun r -> PwMgrNetInput (cb r))), x)) (fun s x ->
    StackPush ((PwMgrDecrypt s), x))

(** val mkPwMgrStack :
    ('a1 input, 'a1) systemActions -> (__ -> (char list -> __ action) ->
    (char list -> __ action) -> (uiInput, __) process) -> (__ -> __ ->
    (char list -> (char list * char list) list -> ((__, httpResponse)
    abortable -> __ netInput) -> __ action) -> (char list -> __ action) ->
    (__ netInput, __) process) -> ('a1 input, 'a1) stackProcess **)

let mkPwMgrStack sys ui0 net0 =
  pwMgrLoop sys (wrap_ui sys ui0) (wrap_net sys net0)

(** val pwMgrStack :
    ('a1 input, 'a1) systemActions -> ('a1 input, 'a1) stackProcess **)

let pwMgrStack sys =
  mkPwMgrStack sys (fun _ -> ui) (fun _ _ -> net)

(** val proc :
    ('a1 input, 'a1) systemActions -> 'a1 action * ('a1 input, 'a1) process **)

let proc sys =
  ((sys.consoleIn (fun x -> PwMgrConsoleIn x)),
    (runStackProcess (pwMgrStack sys)))

