type __ = Obj.t

type ('a, 'b) sum =
| Inl of 'a
| Inr of 'b

val fst : ('a1 * 'a2) -> 'a1

val snd : ('a1 * 'a2) -> 'a2

val app : 'a1 list -> 'a1 list -> 'a1 list

val id : 'a1 -> 'a1

type 'a sig0 =
  'a
  (* singleton inductive, whose constructor was exist *)

type 'world action = 'world -> 'world

type ('input, 'world) process = ('input, 'world) __process Lazy.t
and ('input, 'world) __process =
| Step of ('input -> 'world action * ('input, 'world) process)

type ('input, 'world) stackWorld =
| StackDone
| StackPush of 'input * ('input, 'world) stackWorld
| StackLift of 'world action * ('input, 'world) stackWorld

type ('input, 'world) stackProcess =
  ('input, ('input, 'world) stackWorld) process

val stepStackProcess :
  (('a1, 'a2) stackWorld * ('a1, 'a2) stackProcess) -> 'a2 action * ('a1,
  'a2) stackProcess

val runStackProcess : ('a1, 'a2) stackProcess -> ('a1, 'a2) process

val map : ('a1 -> 'a2) -> 'a1 list -> 'a2 list

val flat_map : ('a1 -> 'a2 list) -> 'a1 list -> 'a2 list

val fold_right : ('a2 -> 'a1 -> 'a1) -> 'a1 -> 'a2 list -> 'a1

val filter : ('a1 -> bool) -> 'a1 list -> 'a1 list

val find : ('a1 -> bool) -> 'a1 list -> 'a1 option

val string_dec : char list -> char list -> bool

val append : char list -> char list -> char list

type ('world, 't) abortable =
| Started of 'world action
| Done of 't

type httpStatus =
| HttpOk
| HttpPreconditionFailed
| HttpUnrecognizedCode

type httpResponse = { httpResponseStatus : httpStatus;
                      httpResponseStatusText : char list;
                      httpResponseProtocol : char list;
                      httpResponseHeader : (char list * char list) list;
                      httpResponseBody : char list }

val httpResponseStatus : httpResponse -> httpStatus

val httpResponseBody : httpResponse -> char list

type ('input, 'world) systemActions = { consoleErr : (char list -> 'world
                                                     action);
                                        consoleIn : ((char list -> 'input) ->
                                                    'world action);
                                        consoleOut : (char list -> 'world
                                                     action);
                                        exit : (Big.big_int -> 'world action);
                                        getArgv : ((char list list -> 'input)
                                                  -> 'world action);
                                        getNanosecs : ((Big.big_int ->
                                                      'input) -> 'world
                                                      action);
                                        getRandomness : (Big.big_int ->
                                                        (char list -> 'input)
                                                        -> 'world action);
                                        httpPOST : (char list ->
                                                   (char list * char list)
                                                   list -> (('world,
                                                   httpResponse) abortable ->
                                                   'input) -> 'world action);
                                        setDebug : (Big.big_int -> 'world
                                                   action);
                                        sleepNanosecs : (Big.big_int ->
                                                        'input -> 'world
                                                        action) }

val consoleIn : ('a1, 'a2) systemActions -> (char list -> 'a1) -> 'a2 action

val consoleOut : ('a1, 'a2) systemActions -> char list -> 'a2 action

val httpPOST :
  ('a1, 'a2) systemActions -> char list -> (char list * char list) list ->
  (('a2, httpResponse) abortable -> 'a1) -> 'a2 action

val storageURI : char list

val storageSet :
  (char list -> (char list * char list) list -> (('a1, httpResponse)
  abortable -> 'a3) -> 'a2 action) -> char list -> char list -> char list ->
  (('a1, (char list option, httpResponse) sum) abortable -> 'a3) -> 'a2
  action

type uiInput =
| UiConsoleIn of char list
| UiDecrypted of char list

val split : char -> char list -> char list list

val newline : char

val dump : (char list * char list) list -> char list

val load : char list -> (char list * char list) list

val uiLoopBody :
  (char list -> 'a1 action) -> (char list -> 'a1 action) ->
  ((char list * char list) list -> (uiInput, 'a1) process) ->
  (char list * char list) list -> uiInput -> 'a1 action * (uiInput, 'a1)
  process

val uiLoop :
  (char list -> 'a1 action) -> (char list -> 'a1 action) ->
  (char list * char list) list -> (uiInput, 'a1) process

val ui :
  (char list -> 'a1 action) -> (char list -> 'a1 action) -> (uiInput, 'a1)
  process

type 'world0 netInput =
| NetReceived of ('world0, (char list option, httpResponse) sum) abortable
| NetEncrypted of char list

val storageId : char list

val net :
  (char list -> (char list * char list) list -> (('a1, httpResponse)
  abortable -> 'a1 netInput) -> 'a2 action) -> (char list -> 'a2 action) ->
  ('a1 netInput, 'a2) process

val getStep : ('a1, 'a2) process -> 'a1 -> 'a2 action * ('a1, 'a2) process

type 'world input =
| PwMgrConsoleIn of char list
| PwMgrNetInput of 'world netInput
| PwMgrEncrypt of char list
| PwMgrDecrypt of char list

val pwMgrLoopBody :
  ('a1 input, 'a1) systemActions -> ((uiInput, ('a1 input, 'a1) stackWorld)
  process -> ('a1 netInput, ('a1 input, 'a1) stackWorld) process -> ('a1
  input, 'a1) stackProcess) -> (uiInput, ('a1 input, 'a1) stackWorld) process
  -> ('a1 netInput, ('a1 input, 'a1) stackWorld) process -> 'a1 input -> ('a1
  input, 'a1) stackWorld action * ('a1 input, 'a1) stackProcess

val pwMgrLoop :
  ('a1 input, 'a1) systemActions -> (uiInput, ('a1 input, 'a1) stackWorld)
  process -> ('a1 netInput, ('a1 input, 'a1) stackWorld) process -> ('a1
  input, 'a1) stackProcess

val wrap_ui :
  ('a1 input, 'a1) systemActions -> (__ -> (char list -> __ action) ->
  (char list -> __ action) -> (uiInput, __) process) -> (uiInput, ('a1 input,
  'a1) stackWorld) process

val wrap_net :
  ('a1 input, 'a1) systemActions -> (__ -> __ -> (char list ->
  (char list * char list) list -> ((__, httpResponse) abortable -> __
  netInput) -> __ action) -> (char list -> __ action) -> (__ netInput, __)
  process) -> ('a1 netInput, ('a1 input, 'a1) stackWorld) process

val mkPwMgrStack :
  ('a1 input, 'a1) systemActions -> (__ -> (char list -> __ action) ->
  (char list -> __ action) -> (uiInput, __) process) -> (__ -> __ ->
  (char list -> (char list * char list) list -> ((__, httpResponse) abortable
  -> __ netInput) -> __ action) -> (char list -> __ action) -> (__ netInput,
  __) process) -> ('a1 input, 'a1) stackProcess

val pwMgrStack :
  ('a1 input, 'a1) systemActions -> ('a1 input, 'a1) stackProcess

val proc :
  ('a1 input, 'a1) systemActions -> 'a1 action * ('a1 input, 'a1) process

